# -*-coding: utf-8 -*-
"""
    @Project: python-learning-notes
    @File   : numpy_tools.py
    @Author : panjq
    @E-mail : pan_jinquan@163.com
    @Date   : 2019-10-23 12:01:36
"""
import numpy as np
import math
from sklearn import metrics, preprocessing


def matching_data_vecror(data, vector):
    '''
    从data中匹配vector向量，查找出现vector的index,如：
    data = [[1., 0., 0.],[0., 0., 0.],[2., 0., 0.],
            [0., 0., 0.],[0., 3., 0.],[0., 0., 4.]]
    # 查找data中出现[0, 0, 0]的index
    data = np.asarray(data)
    vector=[0, 0, 0]
    index =matching_data_vecror(data,vector)
    print(index)
    >>[False  True False  True False False]
    # 实现去除data数组中元素为[0, 0, 0]的行向量
    pair_issame_1 = data[~index, :]  # 筛选数组
    :param data:
    :param vector:
    :return:
    '''
    # index = (data[:, 0] == 0) & (data[:, 1] == 0) & (data[:, 2] == 0)
    row_nums = len(data)
    clo_nums = len(vector)
    index = np.asarray([True] * row_nums)
    for i in range(clo_nums):
        index = index & (data[:, i] == vector[i])
    return index


def set_mat_vecror(data, index, vector):
    '''
    实现将data指定index位置的数据设置为vector
    # 实现将大于阈值分数的point，设置为vector = [10, 10]
    point = [[0., 0.], [1., 1.], [2., 2.],
             [3., 3.], [4., 4.], [5., 5.]]
    point = np.asarray(point) # 每个数据点
    score = np.array([0.7, 0.2, 0.3, 0.4, 0.5, 0.6])# 每个数据点的分数
    score_th=0.5
    index = np.where(score > score_th) # 获得大于阈值分数的所有下标
    vector = [10, 10]                  # 将大于阈值的数据设置为vector
    out = set_mat_vecror(point, index, vector)
    :param data:
    :param index:
    :param vector:
    :return:
    '''
    data[index, :] = vector
    return data


def find_max_shape_data(list_data):
    max_shape_data = np.asarray([])
    for data in list_data:
        if len(max_shape_data) < len(data):
            max_shape_data = data
    return max_shape_data


def data_alignment(data):
    '''
    row_stack()函数扩展行，column_stack()函数扩展列
    :param list_data:
    :param align:
    :param extend:
    :return:
    '''
    max_shape_data = find_max_shape_data(data)
    for i in range(len(data)):
        maxdata = np.zeros(shape=max_shape_data.shape, dtype=max_shape_data.dtype) - 1
        shape = data[i].shape
        if len(shape) == 1:
            maxdata[0:shape[0]] = data[i]
        else:
            maxdata[0:shape[0], 0:shape[1]] = data[i]
        data[i] = maxdata
    # data = np.asarray(data)
    return data


def get_batch(image_list, batch_size):
    '''
    batch size data
    :param image_list:
    :param batch_size:
    :return:
    '''
    sample_num = len(image_list)
    batch_num = math.ceil(sample_num / batch_size)

    for i in range(batch_num):
        start = i * batch_size
        end = min((i + 1) * batch_size, sample_num)
        batch_image = image_list[start:end]
        print("batch_image:{}".format(batch_image))


def gen_range(shape=None, start=None, *args, **kwargs):
    '''
    create range data->
    gen_range(shape=(10, 10), start=0, stop=100)
    :param shape:
    :param start:
    :param args:
    :param kwargs:
    :return:
    '''
    data = np.arange(start, *args, **kwargs)
    if shape:
        data = np.reshape(data, newshape=shape)
    return data


def mat2d_data(data, indexes):
    '''
    get element by indexes
    data=numpy_tools.gen_range((3,3),0,9)
    data=np.mat(data)
    indexes=np.asarray([[1,1],[2,1],[2,2]])
    out=mat2d_data(data, indexes)
    print(data)
    print(out)
    :param data:
    :param indexes:
    :return:
    '''
    out = data[indexes[:, 0], indexes[:, 1]]
    return out


def count_sort_list(list_data: list, reverse=True):
    '''
    给定一个非空正整数的数组，按照数组内数字重复出现次数，从高到低排序
    :param list_data:
    :param reverse: True-降序,Fasle-升序
    :return:
    '''
    d = {}
    list_sorted = []
    for i in list_data:
        d[i] = list_data.count(i)
    # 根据字典值的降序排序
    d_sorted = sorted(d.items(), key=lambda x: x[1], reverse=reverse)
    # 输出排序后的数组
    for x in d_sorted:
        for number in range(0, x[1]):
            list_sorted.append(x[0])
    return list_sorted


def remove_list_data(list_data, flag=["", -1]):
    '''
    删除list所有符合条件元素
    :param list_data:
    :param flag:
    :return:
    '''
    for f in flag:
        while f in list_data:
            list_data.remove(f)
    return list_data


def label_alignment(data_list):
    mat = np.asarray(data_list).T
    label_list = []
    for data in mat:
        out = count_sort_list(data.tolist(), reverse=True)
        out = remove_list_data(out, flag=["", -1])
        if out:
            label = out[0]
        else:
            label = -1
        label_list.append(label)
    return label_list


def __print(data, info=""):
    print("-------------------------------------")
    print(info)
    for index in range(len(data)):
        print("{}".format(data[index]))


def norm(x, p=1, axis=0):
    """L-p范数"""
    y = np.linalg.norm(x, ord=p, axis=axis, keepdims=True)
    return y


def l2(x, axis=0):
    """L-2范数"""
    y = np.sum(np.square(x), axis=axis)
    return y


def l1(x, axis=0):
    """L-1范数"""
    y = np.sum(np.abs(x), axis=axis)
    return y


def L1_loss(y_true, y_pre):
    return np.sum(np.abs(y_true - y_pre))


def L2_loss(y_true, y_pre):
    return np.sum(np.square(y_true - y_pre))


def mean_squared_error(y_true, y_pre):
    """MSE(Mean Squared Error)平均平方误差(L2)"""
    l2 = np.sum(np.square(y_true - y_pre))
    return l2 / y_true.size


def mean_absolute_error(y_true, y_pre):
    """MAE(Mean Absolute Error)平均绝对差值(L1),也等于MAD(Mean Absolute Difference)"""
    l1 = np.sum(np.abs(y_true - y_pre))
    return l1 / y_true.size


def mean(data):
    return np.mean(data)


def var(data):
    # 求方差
    return np.var(data)


def std(data):
    # 求标准差
    return np.std(data, ddof=1)


def load_data(data_path):
    return np.load(data_path)


def save_bin(data, bin_file, dtype="double"):
    """
    https://www.cnblogs.com/yaos/p/12105108.html
    C++int对应Python np.intc
    C++float对应Python np.single
    C++double对应Python np.double
    :param data:
    :param bin_file:
    :param dtype:
    :return:
    """
    data = data.astype(np.double)
    data.astype(dtype).tofile(bin_file)


def load_bin(bin_file, shape=None, dtype="double"):
    """
    ==================python load bin data ================
    bin_file = "data.bin"
    shape = (2, 5)
    data1 = np.arange(10, 20).reshape(shape)
    save_bin(data1, bin_file)
    data2 = load_bin(bin_file, shape)
    print(data1)
    print(data2)
    ===================C++ load bin data ===================
    #include <iostream>
    #include <fstream>
    using namespace std;
    int main()
    {
      int row=2;
      int col=5;
      double fnum[row][col] = {0};
      ifstream in("bin/data.bin", ios::in | ios::binary);
      in.read((char *) &fnum, sizeof fnum);
      cout << in.gcount() << " bytes read\n";
      // show values read from file
      for(int i=0; i<row; i++){
          for(int j=0;j<col;j++){
                cout << fnum[i][j] << ",";
          }
           std::cout<<endl;
      }
      in.close();
      return 0;
    }
    >>result:
    80 bytes read
    10,11,12,13,14,
    15,16,17,18,19,
    =======================================================
    :param bin_file:
    :param dtype:
    :return:
    """
    data = np.fromfile(bin_file, dtype=dtype)
    if shape:
        data = np.reshape(data, shape)
    return data


def get_mat_argmax(data):
    """
    寻找置信度最高的那个下标

     data= [[30 54 25]
            [40 40 89]
            [29 36 25]]
    index,v = get_mat_argmax(data)
    ===============
    index = [1, 2]
      v  = 89
    :param data:
    :return: index=[y,x]
             v= max val
    """

    cols = data.shape[1]
    loc = np.argmax(data)
    y = loc // cols
    x = loc % cols
    v = data[y, x]
    return [y, x], v


def get_mat_argmin(data):
    """
    寻找置信度最高的那个下标

     data= [[30 54 25]
            [40 40 89]
            [29 36 25]]
    index,v = get_mat_argmax(data)
    ===============
    index = [1, 2]
      v  = 89
    :param data:
    :return: index=[y,x]
             v= max val
    """

    cols = data.shape[1]
    loc = np.argmin(data)
    y = loc // cols
    x = loc % cols
    v = data[y, x]
    return [y, x], v


def rmse(data1, data2):
    '''
    均方差
    :param predictions:
    :param targets:
    :return:
    '''
    return np.sqrt(((data1 - data2) ** 2).mean())


def get_error(data1, data2):
    """
    MSE（均方误差）、RMSE （均方根误差）、MAE （平均绝对误差）
    :param data1:
    :param data2:
    :return:
    """
    mse = metrics.mean_squared_error(data1, data2)
    rmse = np.sqrt(mse)
    mae = metrics.mean_absolute_error(data1, data2)
    return mse, rmse, mae


class Preprocessing(object):
    """
    参考：https://www.cnblogs.com/jojo123/p/6770340.html
    PS：  输入数据x表示样本特征矩阵，其中每行是一个样本，每列是一个相同的特征属性
          一般数据处理都是针对属性特征(列)进行操作的
    """

    @staticmethod
    def scale(x, **kwargs):
        """
        功能：标准化:(x-mean)/std
        说明：将数据按其属性(按列进行)减去其均值，然后除以其方差。
              最后得到的结果是，对每个属性/每列来说所有数据都聚集在0附近，方差值为1。
        """
        y = preprocessing.scale(x, **kwargs)
        # mean = x.mean(axis=0)  # calculate mean
        # std = x.std(axis=0)  # calculate variance
        # y = (x - mean) / std  # standardize X
        return y

    @staticmethod
    def minmax_scaler(x, feature_range=(0, 1), **kwargs):
        """
        功能：最大最小值归一化
        说明：将属性缩放到一个指定的最大值和最小值(通常是1-0)之间
        """
        scaler = preprocessing.MinMaxScaler(feature_range=feature_range, **kwargs)
        y = scaler.fit_transform(x)
        # y = Preprocessing.minmax_normalization(x, omin=feature_range[0], omax=feature_range[1])
        return y

    @staticmethod
    def minmax_normalization(x, omin=0.0, omax=1.0, imin=None, imax=None):
        """
        功能：NORMALIZATION 将数据x归一化到任意区间[ymin,omax]范围的方法
        :param x:  输入参数x：需要被归一化的数据,numpy
        :param omin: 输入参数omin：归一化的区间[omin,omax]下限
        :param omax: 输入参数ymax：归一化的区间[omin,omax]上限
        :param imin: 输入参数imin的最小值
        :param imax: 输入参数ymax的最大值
        :return: 输出参数y：归一化到区间[omin,omax]的数据
        """
        imax = imax if imax is not None else np.max(x, axis=0)  # 计算每列最大值
        imin = imin if imin is not None else np.min(x, axis=0)  # 计算每列最小值
        y = (omax - omin) * (x - imin) / (imax - imin) + omin
        return y

    @staticmethod
    def normalization(x, norm='l2', **kwargs):
        """
        功能：正则化
        说明：对每个样本计算其p-范数，然后对该样本中每个元素除以该范数，
              这样处理的结果是使得每个处理后样本的p-范数(l1-norm,l2-norm)等于1(每行和等于1)
        """
        y = preprocessing.normalize(x, norm=norm, **kwargs)
        # norms = np.sqrt(np.sum(np.square(x), axis=1))
        # y = x / norms[:, np.newaxis]
        return y


if __name__ == "__main__":
    X = np.array([[1., -1., 2.],
                  [2., 0., 0.],
                  [0., 1., -1.]])
    # X_scale = Preprocessing.minmax_scaler(X)
    X_scale = Preprocessing.normalization(X)
    print(X_scale)


# -*-coding: utf-8 -*-
"""
    @Project: PyKinect2-OpenCV
    @File   : geometry_tools.py
    @Author : panjq
    @E-mail : pan_jinquan@163.com
    @Date   : 2019-10-11 09:23:21
"""
# -*- coding: utf-8 -*-

import numpy as np


def compute_point2point_distance(area_point, target_point):
    """
    计算点到中心点的距离，取area_point的中心点
    :param area_point:
    :param target_point:
    :return:
    """
    # point1 = area_point[0, :]
    # point2 = area_point[1, :]
    # point3 = area_point[2, :]
    mean_point = np.mean(area_point, axis=0)
    d = np.sqrt(np.sum(np.square(mean_point - target_point)))
    # d = np.linalg.norm(point1 - target_point)
    return d


def compute_distance(vector1, vector2):
    """
    计算两个vector的距离
    :param vector1:
    :param vector2:
    :return:
    """
    d = np.sqrt(np.sum(np.square(vector1 - vector2)))
    # d = np.linalg.norm(vector1 - vector2)
    return d


def compute_point2area_distance(area_point, target_point):
    """
    计算点到平面的距离，平面由point1, point2, point3确定
    :param area_point:
    :param target_point:
    :return:
    """
    point1 = area_point[0, :]
    point2 = area_point[1, :]
    point3 = area_point[2, :]
    point4 = target_point
    d = point2area_distance(point1, point2, point3, point4)
    return d


def define_area(point1, point2, point3):
    """
    法向量    ：n={A,B,C}
    空间上某点：p={x0,y0,z0}
    点法式方程：A(x-x0)+B(y-y0)+C(z-z0)=Ax+By+Cz-(Ax0+By0+Cz0)
    https://wenku.baidu.com/view/12b44129af45b307e87197e1.html
    :param point1:
    :param point2:
    :param point3:
    :param point4:
    :return:（Ax, By, Cz, D）代表：Ax + By + Cz + D = 0
    """
    point1 = np.asarray(point1)
    point2 = np.asarray(point2)
    point3 = np.asarray(point3)
    AB = np.asmatrix(point2 - point1)
    AC = np.asmatrix(point3 - point1)
    N = np.cross(AB, AC)  # 向量叉乘，求法向量
    # Ax+By+Cz
    Ax = N[0, 0]
    By = N[0, 1]
    Cz = N[0, 2]
    D = -(Ax * point1[0] + By * point1[1] + Cz * point1[2])
    return Ax, By, Cz, D


def define_line(point1, point2):
    '''
    定义线
    y-y1=k(x-x1),k=(y2-y1)/(x2-x1)=>
    kx-y+(y1-kx1)=0 <=> Ax+By+C=0
    => A=K=(y2-y1)/(x2-x1)
    => B=-1
    => C=(y1-kx1)
    :param point1:
    :param point2:
    :return:
    '''
    x1, y1 = point1[0], point1[1]
    x2, y2 = point2[0], point2[1]
    A = (y2 - y1) / (x2 - x1)  # K
    B = -1
    C = y1 - A * x1
    return A, B, C


def point2line_distance(point1, point2, target_point):
    '''
    计算点到线的距离，线由point1, point2确定
    :param point1: line point1
    :param point2: line point2
    :param target_point: target_point
    :return:
    '''
    A, B, C = define_line(point1, point2)
    mod_d = A * target_point[0] + B * target_point[1] + C
    mod_sqrt = np.sqrt(np.sum(np.square([A, B])))
    d = abs(mod_d) / mod_sqrt
    return d


def point2area_distance(point1, point2, point3, point4):
    """
    :param point1:数据框的行切片，三维
    :param point2:
    :param point3:
    :param point4:
    :return:点到面的距离
    """
    Ax, By, Cz, D = define_area(point1, point2, point3)
    mod_d = Ax * point4[0] + By * point4[1] + Cz * point4[2] + D
    mod_area = np.sqrt(np.sum(np.square([Ax, By, Cz])))
    d = abs(mod_d) / mod_area
    return d


def create_vector(point1, point2):
    '''
    P12 = point2-point1
    :param point1:
    :param point2:
    :return:
    '''
    if not isinstance(point1, np.ndarray):
        point1 = np.asarray(point1, dtype=np.float32)
    if not isinstance(point2, np.ndarray):
        point2 = np.asarray(point2, dtype=np.float32)
    return point2 - point1


def create_2vectors(P1, P2, Q1, Q2):
    '''
    P12 = P2-P1
    Q21 = Q2-Q1
    :param P1:
    :param P2:
    :param Q1:
    :param Q2:
    :return:
    '''
    v1 = create_vector(P1, P2)
    v2 = create_vector(Q1, Q2)
    return v1, v2


def radian2angle(radian):
    '''弧度->角度'''
    angle = radian * (180 / np.pi)
    return angle


def angle2radian(angle):
    '''角度 ->弧度'''
    radian = angle * np.pi / 180.0
    return radian


def compute_point_angle(P1, P2, Q1, Q2, minangle=True):
    x, y = create_2vectors(P1, P2, Q1, Q2)
    angle = compute_vector_angle(x, y, minangle=minangle)
    return angle


def compute_horizontal_angle(P1, P2, minangle=False):
    """
    计算顺时针(向上为正，向下为负),水平角度大小,
    注意图像坐标与几何坐标Y轴上下相反的
    :param P1:
    :param P2:
    :param Q1:
    :param Q2:
    :param minangle:
    :return: angle<0表示从左往右向下倾斜;angle>0表示从左往右向上倾斜
    """
    Q1 = (0, 0)
    Q2 = (1, 0)
    x, y = create_2vectors(P1, P2, Q1, Q2)
    angle = compute_vector_angle(x, y, minangle=minangle)
    if x[1] > 0:
        # 在第三和四象限
        angle = -angle
    return angle


def compute_vector_angle(a, b, minangle=True):
    '''
    cosφ = u·v/|u||v|
    https://wenku.baidu.com/view/301a6ba1250c844769eae009581b6bd97f19bca3.html?from=search
    :param a:
    :param b:
    :return:
    '''
    # 两个向量
    x = np.array(a)
    y = np.array(b)
    Lx = np.sqrt(x.dot(x))
    Ly = np.sqrt(y.dot(y))
    value = x.dot(y) / ((Lx * Ly) + 1e-6)  # cosφ = u·v/|u||v|
    value = np.clip(value, -1, 1)
    radian = np.arccos(value)
    angle = radian2angle(radian)
    if minangle:
        # angle = np.where(angle > 90, 180 - angle, angle)
        angle = angle if angle < 90 else 180 - angle
    return angle


def line_test():
    '''
    angle: 56.789092174788685
    radian: 0.9911566376686096
    cosφ = u·v/|u||v|
    :return:
    '''
    # 两个向量
    point1 = np.array([1, 1, 0.5], dtype=np.float32)
    point2 = np.array([0.5, 0, 1], dtype=np.float32)
    point3 = np.array([1, 0, 0], dtype=np.float32)
    point4 = np.array([0.5, 0, 1], dtype=np.float32)
    angle = compute_point_angle(point1, point2, point3, point4)
    radian = angle2radian(angle)
    print("angle:", angle)
    print("radian:", radian)


def image2plane_coordinates(image_points, height=0):
    """
    将图像坐标转换到平面坐标或者将图像坐标(x,y)转换到平面坐标(x`,y`)：
    :param image_points:
    :param height: height=0 (默认值)： 表示平面原点在图像左上角
    :param         height=image.height： 表示平面原点在图像左上下角
    :return:
    """
    if not isinstance(image_points, np.ndarray):
        points = np.asarray(image_points)
    else:
        points = image_points.copy()
    points[:, 1] = height - points[:, 1]
    return points


def calculate_camera_distance(P, W, F):
    """
    https://blog.csdn.net/m0_37811342/article/details/80394935
    已知摄像头的焦距F和目标物体的尺寸大小W，求目标到摄像机的距离D
    D = W*F/P，其中：
        Ｆ是摄像机焦距,单位为像素Pixel
        Ｐ是物体的像素宽度,单位为像素Pixel
        Ｗ是物体的实际宽度,单位为cm或者m
        D是目标到摄像机的距离D，单位与Ｗ保持一致
    :param P:目标在像素坐标中的宽度
    :param W:目标在已知世界坐标的宽度
    :param F:相机的焦距，单位为Pixel
    :return: 返回目标在世界坐标中距离相机的距离
    """
    D = (W * F) / P
    return D


def calculate_camera_focal_length(P, D, W):
    """
    计算相机焦距 F = P*D/W，其中：
        Ｆ是摄像机焦距,单位为像素Pixel
        Ｐ是物体的像素宽度,单位为像素Pixel
        Ｗ是物体的实际宽度,单位为cm或者m
        D是目标到摄像机的距离D，单位与Ｗ保持一致
    :param P:
    :param D:
    :param W:
    :return:
    """
    F = (P * D) / W  # 单位为像素
    return F


def rotate_point(point1, point2, angle, height):
    """
    点point1绕点point2旋转angle后的点
    ======================================
    在平面坐标上，任意点P(x1,y1)，绕一个坐标点Q(x2,y2)旋转θ角度后,新的坐标设为(x, y)的计算公式：
    x= (x1 - x2)*cos(θ) - (y1 - y2)*sin(θ) + x2 ;
    y= (x1 - x2)*sin(θ) + (y1 - y2)*cos(θ) + y2 ;
    ======================================
    将图像坐标(x,y)转换到平面坐标(x`,y`)：
    x`=x
    y`=height-y
    :param point1:
    :param point2: base point (基点)
    :param angle: 旋转角度，正：表示逆时针，负：表示顺时针
    :param height: 图像的height
    :return:
    """
    x1, y1 = point1
    x2, y2 = point2
    # 将图像坐标转换到平面坐标
    y1 = height - y1
    y2 = height - y2
    x = (x1 - x2) * np.cos(np.pi / 180.0 * angle) - (y1 - y2) * np.sin(np.pi / 180.0 * angle) + x2
    y = (x1 - x2) * np.sin(np.pi / 180.0 * angle) + (y1 - y2) * np.cos(np.pi / 180.0 * angle) + y2
    # 将平面坐标转换到图像坐标
    y = height - y
    return (x, y)


def points_interpolate(points, num=-1):
    """
    对曲线的坐标点进行插值
    :param points:坐标点(x,y)数组
    :param num: 插值个数，默认2倍插值
    :return:
    """
    from scipy.interpolate import make_interp_spline

    if num <= 0: num = 2 * len(points)
    x = points[:, 0]
    y = points[:, 1]
    sx = np.linspace(min(x), max(x), num=num)
    sy = make_interp_spline(x, y)(sx)
    out = np.vstack((sx, sy)).T
    return out


def points_smoothing(points, winsize=3):
    """
    对曲线的坐标点进行平滑
    :param points:坐标点(x,y)数组
    :param winsize: 平滑的窗口
    :return:
    """
    r = winsize // 2
    out = points.copy()
    num = len(out)
    for i in range(0, num):
        s = max(0, i - r)
        e = min(num, i + r)
        out[i] = np.mean(points[s:e], axis=0)
    return out


def rotate_points(points, centers, angle, height):
    """
    eg.:
    height, weight, d = image.shape
    point1 = [[300, 200],[50, 200]]
    point1 = np.asarray(point1)
    center = [[200, 200]]
    point3 = rotate_points(point1, center, angle=30, height=height)
    :param points:
    :param centers:
    :param angle:
    :param height:
    :return:
    """
    if not isinstance(points, np.ndarray):
        points = np.asarray(points)
    if not isinstance(centers, np.ndarray):
        centers = np.asarray(centers)
    dst_points = points.copy()
    # 将图像坐标转换到平面坐标
    dst_points[:, 1] = height - dst_points[:, 1]
    centers[:, 1] = height - centers[:, 1]
    x = (dst_points[:, 0] - centers[:, 0]) * np.cos(np.pi / 180.0 * angle) - (
            dst_points[:, 1] - centers[:, 1]) * np.sin(np.pi / 180.0 * angle) + centers[:, 0]
    y = (dst_points[:, 0] - centers[:, 0]) * np.sin(np.pi / 180.0 * angle) + (
            dst_points[:, 1] - centers[:, 1]) * np.cos(np.pi / 180.0 * angle) + centers[:, 1]
    # 将平面坐标转换到图像坐标
    y = height - y
    dst_points[:, 0] = x
    dst_points[:, 1] = y
    return dst_points


def demo_for_rotate_point():
    import cv2
    from utils import image_utils

    image_path = "4.jpg"
    image = cv2.imread(image_path)
    image = image_utils.resize_image(image, resize_width=800, resize_height=800)
    height, weight, d = image.shape
    point1 = [[300, 200], [50, 200]]
    point1 = np.asarray(point1)
    # center = [[200, 200]]
    center = [(weight / 2.0, height / 2.0)]
    for i in range(360):
        point2 = rotate_points(point1, center, angle=i, height=height)
        image_vis = image_utils.draw_points_text(image, center, texts=["center"], drawType="simple")
        image_vis = image_utils.draw_points_text(image_vis, point1, texts=[str(i)] * len(point1),
                                                 drawType="simple")
        image_vis = image_utils.draw_points_text(image_vis, point2, texts=[str(i)] * len(point1),
                                                 drawType="simple")
        image_utils.cv_show_image("image", image_vis)


def line_test():
    '''
    angle: 56.789092174788685
    radian: 0.9911566376686096
    cosφ = u·v/|u||v|
    :return:
    '''
    # 两个向量
    point1 = np.array([1, 1, 0.5], dtype=np.float32)
    point2 = np.array([0.5, 0, 1], dtype=np.float32)
    point3 = np.array([1, 0, 0], dtype=np.float32)
    point4 = np.array([0.5, 0, 1], dtype=np.float32)
    angle = compute_point_angle(point1, point2, point3, point4)
    radian = angle2radian(angle)
    print("angle:", angle)
    print("radian:", radian)


def line_test2():
    '''
    angle: 56.789092174788685
    radian: 0.9911566376686096
    cosφ = u·v/|u||v|
    :return:
    '''
    # 两个向量
    point1 = np.array([0, 0], dtype=np.float32)
    point2 = np.array([1, 1], dtype=np.float32)
    point3 = np.array([0, 0], dtype=np.float32)
    point4 = np.array([1, 1], dtype=np.float32)
    v1 = create_vector(point1, point2)
    v2 = create_vector(point3, point4)
    angle = compute_vector_angle(v1, v2, minangle=True)
    print("angle:", angle)


if __name__ == '__main__':
    # line_test()
    # line_test2()
    print(radian2angle(60))



# -*- coding:utf-8 -*-
"""
提供工具函数的模块
"""

import logging
import yaml

log = logging.getLogger(__name__)


class Dict2Obj:
    '''
    dict转类对象
    '''

    def __init__(self, args):
        self.__dict__.update(args)


def load_config(config_file='config.yaml'):
    """
    读取配置文件，并返回一个python dict 对象
    :param config_file: 配置文件路径
    :return: python dict 对象
    """
    with open(config_file, 'r', encoding="UTF-8") as stream:
        try:
            config = yaml.load(stream, Loader=yaml.FullLoader)
            # config = Dict2Obj(config)
        except yaml.YAMLError as e:
            print(e)
            return None
    return config


def print_dict(dict_data, save_path=None):
    list_config = []
    print("--" * 30)
    for key in dict_data:
        info = "{}: {}".format(key, dict_data[key])
        print(info)
        list_config.append(info)
    if save_path is not None:
        with open(save_path, "w") as f:
            for info in list_config:
                f.writelines(info + "\n")
    print("--" * 30)


if __name__ == '__main__':
    pass



# -*-coding: utf-8 -*-
"""
    @Author : panjq
    @E-mail : pan_jinquan@163.com
    @Date   : 2022-03-22 09:11:35
    @Brief  :
"""
import sys
import os
import cv2
import base64
import numpy as np
from typing import Any


def base642image(image_base64, prefix="image", use_rgb=False) -> np.ndarray:
    """
    將二进制字符串解码为图像
    :param image_base64: 二进制字符串图像
    :param prefix: base64字符串前缀,用于表识字符串的类型
    :param use_rgb: True:返回RGB的图像, False:返回BGR格式的图像
    :return: 返回图像
    """
    image_base64 = image_base64[len(prefix):]
    image_base64 = bytes(image_base64, 'utf-8')
    image = base64.b64decode(image_base64)
    image = np.fromstring(image, np.uint8)
    image = cv2.imdecode(image, cv2.IMREAD_COLOR)
    if use_rgb:
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    return image


def image2base64(image: np.ndarray, prefix="image", use_rgb=False) -> str:
    """
    将图像编码为二进制字符串
    ``` python
        from io import BytesIO
        bgr_img = Image.fromarray(image)
        buff = BytesIO()
        mg.save(buff, format="PNG")
        image_base64 = base64.b64encode(buff.getvalue()).decode("utf-8")
        image_base64 = str(base64.b64encode(image), encoding='utf-8')
    ```
    :param image: 图像
    :param prefix: base64字符串前缀,用于表识字符串的类型
    :param use_rgb: True:输入image是RGB的图像, False:返输入image是BGR格式的图像
    :return: 返回图像
    """
    img = image.copy()
    if len(img.shape) == 3 and use_rgb:
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
    img = cv2.imencode('.jpeg', img)[1]
    image_base64 = prefix + base64.b64encode(img).decode()
    return image_base64


def read_file2base64(file, prefix="image") -> str:
    """
    读取文件,并编码为二进制字符串
    :param file: 文件路径
    :param prefix: base64字符串前缀,用于表识字符串的类型
    :return:返回二进制字符串
    """
    file_base64 = prefix + base64.b64encode(open(file, 'rb').read()).decode()
    return file_base64


def array2base64(data: Any, prefix="image", use_rgb=False) -> Any:
    if isinstance(data, np.ndarray) and data.dtype == np.uint8:
        return image2base64(data, prefix=prefix, use_rgb=use_rgb)
    elif isinstance(data, np.ndarray):
        return data.tolist()
    elif isinstance(data, list):
        for i in range(len(data)):
            data[i] = array2base64(data[i], prefix=prefix, use_rgb=use_rgb)
    elif isinstance(data, dict):
        for k, v in data.items():
            data[k] = array2base64(v, prefix=prefix, use_rgb=use_rgb)
    return data


def base642array(data: Any, prefix="image", use_rgb=False) -> Any:
    if isinstance(data, str) and prefix == data[0:len(prefix)]:
        return base642image(data, prefix=prefix, use_rgb=use_rgb)
    elif isinstance(data, np.ndarray):
        return data.tolist()
    elif isinstance(data, list):
        for i in range(len(data)):
            data[i] = base642array(data[i], prefix=prefix, use_rgb=use_rgb)
    elif isinstance(data, dict):
        for k, v in data.items():
            data[k] = base642array(v, prefix=prefix, use_rgb=use_rgb)
    return data


def cv_show_image(title: str, image: np.ndarray, use_rgb=True, delay=0):
    """
    调用OpenCV显示RGB图片
    :param title: 图像标题
    :param image: 输入是否是RGB图像
    :param use_rgb: True:输入image是RGB的图像, False:返输入image是BGR格式的图像
    :return:
    """
    img = image.copy()
    if img.shape[-1] == 3 and use_rgb:
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)  # 将BGR转为RGB
    # cv2.namedWindow(title, flags=cv2.WINDOW_AUTOSIZE)
    cv2.namedWindow(title, flags=cv2.WINDOW_NORMAL)
    cv2.imshow(title, img)
    cv2.waitKey(delay)
    return img


if __name__ == "__main__":
    file = "/home/dm/project/python-learning-notes/utils/test.jpg"
    bgr1 = cv2.imread(file)
    image_base64 = image2base64(bgr1)
    image_base64 = read_file2base64(file)
    bgr2 = base642image(image_base64, use_rgb=False)
    cv_show_image("bgr1", bgr1, use_rgb=False, delay=1)
    cv_show_image("bgr2", bgr2, use_rgb=False)
